/**
 * File name ubitx_I2C_V4_02R.ino
 * This source file is under General Public License version 3.
 * 
 * This version uses a built-in Si5351 library
 * Most source code are meant to be understood by the compilers and the computers. 
 * Code that has to be hackable needs to be well understood and properly documented. 
 * Donald Knuth coined the term Literate Programming to indicate code that is written be 
 * easily read and understood.
 * 
 * The Raduino is a small board that includes the Arduino Nano, a 16x2 LCD display and
 * an Si5351a frequency synthesizer. This board is manufactured by Paradigm Ecomm Pvt Ltd
 * 
 * To learn more about Arduino you may visit www.arduino.cc. 
 * 
 * The Arduino works by starts executing the code in a function called setup() and then it 
 * repeatedly keeps calling loop() forever. All the initialization code is kept in setup()
 * and code to continuously sense the tuning knob, the function button, transmit/receive,
 * etc is all in the loop() function. If you wish to study the code top down, then scroll
 * to the bottom of this file and read your way up.
 * 
 * Below are the libraries to be included for building the Raduino 
 * The EEPROM library is used to store settings like the frequency memory, calibration data, 
 * call sign etc .
 *
 *  The main chip which generates up to three oscillators of various frequencies in the
 *  Raduino is the Si5351a. To learn more about Si5351a you can download the datasheet 
 *  from www.silabs.com although, strictly speaking it is not a requirement to understand this code. 
 *  Instead, you can look up the Si5351 library written by xxx, yyy. You can download and 
 *  install it from www.url.com to compile this file.
 *  The Wire.h library is used to talk to the Si5351 and we also declare an instance of 
 *  Si5351 object to control the clocks.
 */

/* 
 *  1. Renamed all files with .cpp extension except ubitx20.ino
 *  2. Creates ubitx20.h to hold constants that are used by multiple files.
 *  3. Created list of external routines and variable not defined in current file.
 *  
 *  This will be the base for further development.
 *  
 *  Ron - W2CTX
 *  Jim - W0EB (editor & beta tester)
 */


#include <Arduino.h>
 
#include <Wire.h>
#include <EEPROM.h>
#include "ubitx.h"



char Version_Text[17] = "uBITX I2C_V4.02R";
char Version_Date[17] = "2018SEP06  15:00";



extern void si5351bx_setfreq(uint8_t clknum, uint32_t fout);
extern void updateDisplay(void);
extern int  btnDown(void);
extern void doMenu(void);
extern int  enc_read(void);
extern void printLine1(char *c);
extern void printLine2(char *c);
extern void printLine3(char *c);
extern void printLine4(char *c);
extern void initOscillators(void);
extern void cwKeyer(void);
extern void menuSplitToggle( void );
extern void menuVfoToggle(void );
extern void Connect_Interrupts(void);
extern void Init_Cat(long baud, int portConfig);
extern void  Process_CAT_Command( void );
extern void doAlignment(void);
extern void DoBaud(void);
extern void menuPaddleSwapToggle( void );
extern void SetIambicA( void );
extern void SetIambicB( void );
extern void SetPaddleSwapN( void);
extern void SetPaddleSwapR( void);


void startTx(byte txMode);
void stopTx(void);

extern bool Volts_Active;
extern volatile bool Ubitx_Voltage_Act;
extern volatile unsigned long Ubitx_Voltage;
extern volatile unsigned long Current_Ubitx_Voltage;
extern int Last_Mode;
extern long ubitx_CAT_Long_Baud_rates[8];
extern int ubitx_baud;
extern int Last_Baud;
extern unsigned int SI5351BX_XTALPF;              // 1:6pf  2:8pf  3:10pf
extern bool Cat_Lock;
extern long CW_TIMEOUT;
extern unsigned long Volt_Divider;

/**
    The main chip which generates up to three oscillators of various frequencies in the
    Raduino is the Si5351a. To learn more about Si5351a you can download the datasheet
    from www.silabs.com although, strictly speaking it is not a requirement to understand this code.

    We no longer use the standard SI5351 library because of its huge overhead due to many unused
    features consuming a lot of program space. Instead of depending on an external library we now use
    Jerry Gaffke's, KE7ER, lightweight standalone minimalist "si5351bx" routines (see further down the
    code). Here are some defines and declarations used by Jerry's routines:
*/


/** 
 * The Raduino board is the size of a standard 16x2 LCD panel. It has three connectors:
 * 
 * First, is an 8 pin connector that provides +5v, GND and six analog input pins that can also be 
 * configured to be used as digital input or output pins. These are referred to as A0,A1,A2,
 * A3,A6 and A7 pins. The A4 and A5 pins are missing from this connector as they are used to 
 * talk to the Si5351 over I2C protocol. 
 * 
 */


/** Get the LCD I2C Library here:
https://bitbucket.org/fmalpartida/new-liquidcrystal/downloads
Move all *_I2C files into a new folder named LiquidCrystal_I2C
in the Arduino library folder, or use the "install zipped library" feature
*/

//now invoke the LCD_I2C library                   (n5ib: 15 Jan 2018)

#include <LiquidCrystal_I2C.h>
//specify the pin connections from the LCD to the PCF8574 chip

//////////////////////////////// user defines their address of theit I2C display
//LiquidCrystal_I2C lcd(0x3F, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);  // 
//LiquidCrystal_I2C lcd(0x2F, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);  //
LiquidCrystal_I2C lcd(0x3F, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);    //

/** arguments for use with PCF8574A I2C I/O Expander chip on LCD I2C interface module:
  I2C 7-bit slave address in hexadecimal, the least significant three bits are user configurable on the interface module
  LCD EN pin to PCF8574A I/O pin P2
  LCD RW pin to PCF8574A I/O pin P1
  LCD RST pin to PCF8574A I/O pin P0
  LCD D4 pin to PCF8574A I/O pin P4
  LCD D5 pin to PCF8574A I/O pin P5
  LCD D6 pin to PCF8574A I/O pin P6
  LCD D7 pin to PCF8574A I/O pin P7
  LCD Backlight pin to PCF8574A I/O pin P3  (optional parameter)
  LCD Backlight control is POSITIVE logic, i.e., "1" turns ON  (optional parameter)
*/

/**
 * The Arduino, unlike C/C++ on a regular computer with gigabytes of RAM, has very little memory.
 * We have to be very careful with variables that are declared inside the functions as they are 
 * created in a memory region called the stack. The stack has just a few bytes of space on the Arduino
 * if you declare large strings inside functions, they can easily exceed the capacity of the stack
 * and mess up your programs. 
 * We circumvent this by declaring a few global buffers as  kitchen counters where we can 
 * slice and dice our strings. These strings are mostly used to control the display or handle
 * the input and output from the USB port. We must keep a count of the bytes used while reading
 * the serial port as we can easily run out of buffer space. This is done in the serial_in_count variable.
 */

//////////////////////////////////////////////

volatile int ubitx_mode;

extern int wpm;

/////////////////////////////////////////////

//////////////////// keyer
unsigned volatile char keyerControl = 0;
unsigned volatile char keyerState = 0;
unsigned volatile char IAMBIC = 0x10;  // 0 for Iambic A, 1 for Iambic B
unsigned volatile char PDLSWAP =   0x00;     // 0x00 for normal, 0x08 for swap

///////////////////
 
char c[21], b[21];      
char printBuff[2][17];  //mirrors what is showing on the two lines of the display
char ritOn = 0;
char splitOn = 0;
char vfoActive = VFO_A;
unsigned long vfoA=07150000L, vfoB=14200000L, sideTone=000, usbCarrier;
unsigned long frequency, ritRxFrequency, ritTxFrequency;  //frequency is the current frequency on the dial
long ritOffset = 0;
long ritOffsetMax = 5000;
unsigned long Freq_Incr[6]={10,100,1000,10000,100000,1000000};
int Freq_Index = 0;

int Freq_Underline[6]={15,14,12,11,10,8};
int Menu_Index = 0;  // new menu
                      

int cwSpeed = 100; //this is actual the dot period in milliseconds
extern int32_t calibration;

/**
 * Raduino needs to keep track of current state of the transceiver. These are a few variables that do it
 */
volatile bool inTx = false;                //it is set to 1 if in transmit mode (whatever the reason : CW, PTT or CAT)

byte menuOn = 0;              //set to 1 when the menu is being displayed, if a menu item sets it to zero, the menu is exited
unsigned char txFilter = 0;   //which of the four transmit filters are in use
boolean modeCalibrate = false;//this mode of menus shows extended menus to calibrate the oscillators and choose the proper
                              //beat frequency
/**
 * Below are the basic functions that control the uBitx. Understanding the functions before 
 * you start hacking around
 */
/**
 * Select the properly TX harmonic filters
 * The four harmonic filters use only three relays
 * the four LPFs cover 30-21 MHz, 18 - 14 MHz, 7-10 MHz and 3.5 to 5 MHz
 * Briefly, it works like this, 
 * - When KT1 is OFF, the 'off' position routes the PA output through the 30 MHz LPF
 * - When KT1 is ON, it routes the PA output to KT2. Which is why you will see that
 *   the KT1 is on for the three other cases.
 * - When the KT1 is ON and KT2 is off, the off position of KT2 routes the PA output
 *   to 18 MHz LPF (That also works for 14 MHz) 
 * - When KT1 is On, KT2 is On, it routes the PA output to KT3
 * - KT3, when switched on selects the 7-10 MHz filter
 * - KT3 when switched off selects the 3.5-5 MHz filter
 * See the circuit to understand this
 */

void setTXFilters(unsigned long freq){
  
  if (freq > 21000000L){  // the default filter is with 35 MHz cut-off
    digitalWrite(TX_LPF_A, 0);
    digitalWrite(TX_LPF_B, 0);
    digitalWrite(TX_LPF_C, 0);
  }
  else if (freq >= 14000000L){ //throw the KT1 relay on, the 30 MHz LPF is bypassed and the 14-18 MHz LPF is allowed to go through
    digitalWrite(TX_LPF_A, 1);
    digitalWrite(TX_LPF_B, 0);
    digitalWrite(TX_LPF_C, 0);
  }
  else if (freq > 7000000L){
    digitalWrite(TX_LPF_A, 1);
    digitalWrite(TX_LPF_B, 1);
    digitalWrite(TX_LPF_C, 0);    
  }
  else {
    digitalWrite(TX_LPF_A, 1);
    digitalWrite(TX_LPF_B, 1);
    digitalWrite(TX_LPF_C, 1);    
  }
}

/**
 * This is the most frequently called function that configures the 
 * radio to a particular frequency, sideband and sets up the transmit filters
 * 
 * The transmit filter relays are powered up only during the tx so they dont
 * draw any current during RX. 
 * 
 * The carrier oscillator of the detector/modulator is permanently fixed at
 * upper sideband. The sideband selection is done by placing the second oscillator
 * either 12 MHz below or above the 45 MHz signal thereby inverting the sidebands 
 * through mixing of the second local oscillator.
 */

void setFrequency(unsigned long xf){
unsigned long f;

  f = xf;
  frequency = xf;
  
  if( ubitx_mode == MODE_CW) f = f - sideTone;
  if( ubitx_mode == MODE_CWR) f = f + sideTone;
  
  setTXFilters(f);
  if( (ubitx_mode == MODE_SWU)||(ubitx_mode == MODE_USB)||(ubitx_mode == MODE_CW)){
    si5351bx_setfreq(2, SECOND_OSC_USB - usbCarrier + f + ritOffset);
    si5351bx_setfreq(1, SECOND_OSC_USB);
  }else{
    si5351bx_setfreq(2, SECOND_OSC_LSB + usbCarrier + f + ritOffset);
    si5351bx_setfreq(1, SECOND_OSC_LSB);
  }
}

/**
 * startTx is called by the PTT, cw keyer and CAT protocol to
 * put the uBitx in TX mode. It takes care of RIT settings, sideband settings
 * Note: In cw mode, doesn't key the radio, only puts it in TX mode
 */
 
void startTx(byte txMode){
  unsigned long tx_freq; 

if( (ubitx_mode == MODE_SWU)||(ubitx_mode == MODE_SWL)){
  printLine2((char *)"No TX in SW Modes");
  delay(500);
}else{
  if( splitOn) tx_freq = vfoB;
  else tx_freq = frequency;
  
  digitalWrite(TX_RX, 1);
  inTx = true;
  
  #ifdef XMIT_LED
    if( MODE_USB || MODE_LSB )digitalWrite(ON_AIR, 1); // light up the LED on NANO's pin 13
  #endif

  
  if (txMode == TX_CW){
    //turn off the second local oscillator and the BFO
    si5351bx_setfreq(0, 0);
    si5351bx_setfreq(1, 0);
    si5351bx_setfreq(2, tx_freq ); 
  }
}
}
void stopTx(void){
  inTx = false;

  digitalWrite(TX_RX, 0);           //turn off the TX
  si5351bx_setfreq(0, usbCarrier);  //set back the carrier oscillator anyway, CW TX switches it off

  #ifdef XMIT_LED
    if( MODE_USB || MODE_LSB )digitalWrite(ON_AIR, 0); // extinguish the LED on NANO's pin 13
  #endif

  setFrequency(frequency);
}

/**
 * ritEnable is called with a frequency parameter that determines
 * what the RX frequency will be
 */
void ritEnable(void){
  if (!ritOn){
    ritOn = 1;
    setFrequency(frequency);
    updateDisplay();
  }
}

// this is called by the RIT menu routine
void ritDisable(void){
  if (ritOn){
    ritOn = 0;
    setFrequency(frequency);
    updateDisplay();
  }
}


/**
 * Basic User Interface Routines. These check the front panel for any activity
 */


void Save_Radio_State( void ){

  if (vfoActive == VFO_A) EEPROM.put(VFO_A, frequency);
  else EEPROM.put(VFO_B, frequency);
  
}



// test button for short or long push
int Test_Button( char * title1, char * title2,char * title3,char * title4 ){
int i;

  //only if the button is pressed
  if (!btnDown()) return 0;
  delay(100);
  if (!btnDown()) return 0; //debounce
  // OK button is definitely pushed so now detect how long it is pressed.
  
  if ( Cat_Lock == true ){
    while( btnDown()) delay(10);
    return -1;
  }else{
    i=0;
    while ( btnDown() ){
    delay(10);
    i++;
    if ( (i > BUTTON_DELAY_1)&&(i < BUTTON_DELAY_2) )printLine2(title1);
    if ( (i > BUTTON_DELAY_2)&&(i < BUTTON_DELAY_3) )printLine2(title2);
    if ( (i > BUTTON_DELAY_3)&&(i < BUTTON_DELAY_4) )printLine2(title3);
    if ( i > BUTTON_DELAY_4) printLine2(title4);
    }
    if ((i > BUTTON_DELAY_1)&&(i < BUTTON_DELAY_2) )return 1;
    if ((i > BUTTON_DELAY_2 )&&(i < BUTTON_DELAY_3) )return 2;
    if ((i > BUTTON_DELAY_3 )&&(i < BUTTON_DELAY_4) )return 3;
    if ( i > BUTTON_DELAY_4) return 4;
    else return -1;
  }
}
  




void checkButton( void ){
int Do_Something;
char Response_Buffer[32];

  if(btnDown()){

    Do_Something = Test_Button((char*)"Menu",(char*)"RIT",(char*)"Baud",(char*)"Save State");
  
    if(Do_Something != 0 ){
      if( Do_Something == 1){
        doMenu();
      }else if(Do_Something == -1 ){
        Freq_Index = Freq_Index - 1;
        if( Freq_Index < 0 )Freq_Index = 5;
        EEPROM.put(FREQ_INCR, Freq_Index);
        delay(200);
        updateDisplay();
        if ( Cat_Lock == true ){
           sprintf(Response_Buffer,"uFI%1d;",Freq_Index);
           Serial.write(Response_Buffer);
        }
      }else if( Do_Something == 2 ){
        if(ritOn){
          ritDisable();
        }else{
          ritEnable();
        }
        updateDisplay();
      }else if( Do_Something == 3 ){
        DoBaud();
        updateDisplay();
      }else if( Do_Something == 4 ){
        Save_Radio_State();
        updateDisplay();
      }
    }
  }
}


void doTuning(){
  int s;
  long rx_freq; 

  s = enc_read();
  if (s){
   // if( (splitOn)&&(vfoActive == VFO_B)) rx_freq = vfoB;
   // else rx_freq = frequency;
    if( vfoActive == VFO_B ) rx_freq = vfoB;
    else rx_freq = vfoA;
    
    if (s > 4)
      rx_freq += Freq_Incr[Freq_Index];
    else if (s >  2)
      rx_freq += Freq_Incr[Freq_Index];
    else if (s >  0)
      rx_freq +=  Freq_Incr[Freq_Index];
    else if (s <  0)
      rx_freq -= Freq_Incr[Freq_Index];
    else if (s < -2)
      rx_freq -= Freq_Incr[Freq_Index];
    else if (s < -4)
      rx_freq -= Freq_Incr[Freq_Index];

    if( rx_freq <  400000l) rx_freq =  400000l;
    if( rx_freq > 30000000l) rx_freq = 30000000l;
      

    if( vfoActive == VFO_B )vfoB = rx_freq;
    else vfoA = rx_freq;
    frequency = rx_freq;
    setFrequency(frequency);
    updateDisplay();
  }
}

/**
 * RIT only steps back and forth by 100 Hz at a time
 */
void doRIT(){
 
  int knob = enc_read();
  long old_freq = ritOffset;

  if (knob){
  if (knob < 0){
  //  frequency -= 100l;
    ritOffset -= 100l;
    if( abs(ritOffset) > ritOffsetMax) ritOffset=-ritOffsetMax;
  }else if (knob > 0){
    //frequency += 100;
    ritOffset += 100;
    if( ritOffset > ritOffsetMax) ritOffset=ritOffsetMax;
  }
  if (old_freq != ritOffset){
    setFrequency(frequency);
    updateDisplay();
  }
}
}


/**
 * The settings are read from EEPROM. The first time around, the values may not be 
 * present or out of range, in this case, some intelligent defaults are copied into the 
 * variables.
 */
void initSettings(){

 Serial.begin(38400); 
  //read the settings from the eeprom and restore them
  //if the readings are off, then set defaults
  EEPROM.get(MASTER_CAL, calibration);
   
  EEPROM.get(USB_CAL, usbCarrier);
  if (usbCarrier > 12010000l || usbCarrier < 11990000l){
    usbCarrier = 11997000l;
    EEPROM.put(USB_CAL, usbCarrier);
    delay(200);
  }
  
  EEPROM.get(XTALPF, SI5351BX_XTALPF);
  if( (SI5351BX_XTALPF < 1)||(SI5351BX_XTALPF > 3)){
    SI5351BX_XTALPF = 2;
    EEPROM.put(XTALPF, SI5351BX_XTALPF);
    delay(200);
  }
  
  EEPROM.get(VFO_A, vfoA);
  if( vfoA <   400000l ) vfoA = 400000l;
  if( vfoA > 30000000l ) vfoA = 30000000l;
  
  EEPROM.get(VFO_B, vfoB);
  if( vfoB <   400000l ) vfoB = 400000l;
  if( vfoB > 30000000l ) vfoB = 30000000l;
  
  EEPROM.get(CW_SIDETONE, sideTone);
  if( sideTone < 100 || sideTone > 2000 ) sideTone = 800;
  
  EEPROM.get(IAMBIC_MODE, wpm );
  IAMBIC = (volatile unsigned char)wpm;
  if( IAMBIC == 0 )SetIambicA();
  else SetIambicB();
  
  EEPROM.get(PDL_SWAP_VAL, wpm );
  PDLSWAP = (volatile unsigned char)wpm;
  if( PDLSWAP == 0 ) SetPaddleSwapN();
  else SetPaddleSwapR();
  
  EEPROM.get(MODE, wpm );
  if( wpm < 0 ) wpm = 0;
  if( wpm > 5 ) wpm = 5;
  ubitx_mode = wpm;
  Last_Mode = wpm;
  
  EEPROM.get(VOLTS,wpm);
  if( wpm == 0 ) Volts_Active = false;
  else  Volts_Active = true;
  
  EEPROM.get(BAUD,ubitx_baud);
  if( ubitx_baud < 0 ) ubitx_baud = 5;
  if( ubitx_baud > 7 ) ubitx_baud = 5;
  Last_Baud = ubitx_baud;
  
  EEPROM.get(CW_SPEED, wpm);
  if( wpm < 5 ) wpm = 5;
  if( wpm > 50 ) wpm = 50;
  cwSpeed = 600 / wpm;
  
  EEPROM.get(FREQ_INCR, Freq_Index);
  if( (Freq_Index < 0) || (Freq_Index > 4)){
    Freq_Index = 2;
    EEPROM.put(FREQ_INCR, Freq_Index);
    delay(200);
  }

  EEPROM.get( VOLT_DIVIDE, Volt_Divider);
  if( (Volt_Divider > 2000) || (Volt_Divider <= 0 )) Volt_Divider = 982;

  EEPROM.get( TX_DELAY, CW_TIMEOUT);
  if( (CW_TIMEOUT > TX_DELAY_MAX) || (Volt_Divider <= TX_DELAY_MIN )) Volt_Divider = 350l;


 
}

void initPorts(){

  analogReference(DEFAULT);

  

  //??
  pinMode(ENC_A, INPUT_PULLUP);
  pinMode(ENC_B, INPUT_PULLUP);
  pinMode(FBUTTON, INPUT_PULLUP);
  pinMode(AB_SPLIT, INPUT_PULLUP);
  pinMode(UBIT_VCC, INPUT );
  
  // init CW digital pins
  pinMode(DIGITAL_DOT, INPUT_PULLUP);
  pinMode(DIGITAL_DASH, INPUT_PULLUP);
  pinMode(DIGITAL_KEY, INPUT_PULLUP);
 
  pinMode(CW_TONE, OUTPUT);  
  digitalWrite(CW_TONE, 0);
  
  pinMode(TX_RX,OUTPUT);
  digitalWrite(TX_RX, 0);

  pinMode(TX_LPF_A, OUTPUT);
  pinMode(TX_LPF_B, OUTPUT);
  pinMode(TX_LPF_C, OUTPUT);
  digitalWrite(TX_LPF_A, 0);
  digitalWrite(TX_LPF_B, 0);
  digitalWrite(TX_LPF_C, 0);

  pinMode(CW_KEY, OUTPUT);
  digitalWrite(CW_KEY, 0);


  #ifdef XMIT_LED
     pinMode(LED_BUILTIN, OUTPUT); //use the LED on D13 as an "On the Air" indicator
     digitalWrite(LED_BUILTIN, 0);  // turn it off to begin with
  #endif
 


  
}



void setup() {

  lcd.begin(16, 2);
  lcd.clear();

  //we print this line so this shows up even if the Raduino 
  //crashes later in the code
  printLine2(Version_Text); 
  printLine1((char*)" W0EB and W2CTX"); 
  delay(2000);

  initSettings();
  initPorts();  
  initOscillators();

  frequency = vfoA;
  setFrequency(vfoA);
  updateDisplay();

  Ubitx_Voltage_Act = false;
  
  keyerState = IDLE;
  keyerControl = IAMBIC; // 0x10 for B Or 0 for A
  
   lcd.cursor();
   
  /////////////////////////////   ALIGNMENT ////////////////////////////////////
  // check if button pushed during init
  // if it is support alignment
  if( btnDown()) doAlignment();
  
 
  Init_Cat(ubitx_CAT_Long_Baud_rates[ubitx_baud], SERIAL_8N1);


  Connect_Interrupts();

 
}

bool AB_SPLIT_btnDown( void ){
  
 if (digitalRead(AB_SPLIT) == HIGH) return false;
  else return true;
 
}


void checkAB_SPLIT( void ){
int i;

  if (AB_SPLIT_btnDown()){
    //only if the button is pressed
    if (!AB_SPLIT_btnDown()) return;
    delay(100);
    if (!AB_SPLIT_btnDown()) return; //debounce
    // OK button is definitely pushed so now detect how long it is pressed.
    i=0;
    while ( AB_SPLIT_btnDown() ){
      delay(10);
      i++;
      if ( i > BUTTON_DELAY_1 ){
        if(splitOn)printLine2((char*)"SPLIT OFF");
        else printLine2((char*)"SPLIT ON");
        lcd.setCursor(0,2);
      }
    }
    if ( i > BUTTON_DELAY_1 )menuSplitToggle();
    else menuVfoToggle();
  }
}



void loop(){ 

  Process_CAT_Command();
  
  //do tasks only when not tranmsitting 
  if (!inTx){
    if( Volts_Active ){ 
      if( Ubitx_Voltage_Act == true ){
        Ubitx_Voltage = analogRead(UBIT_VCC);
        Ubitx_Voltage_Act = false;
        Ubitx_Voltage = analogRead(UBIT_VCC);
        //if( Current_Ubitx_Voltage != Ubitx_Voltage ) Display_Voltage();
        updateDisplay();
      }
    }
    if ( Cat_Lock == false ){
      Process_CAT_Command();
      checkAB_SPLIT();
      Process_CAT_Command();
      checkButton();
      Process_CAT_Command();
      if (ritOn){
        doRIT();
      Process_CAT_Command();
      }else{
       doTuning();
      }
    }else{
      doTuning();
      checkButton();
    }
  }
}
